// RedbearBLENanStarter.ino  Sketch V1.0 27th August 2017
// Using RedBearLabs BLE Nano with BLEPeripheral library V0.4.0
// Using Arduino V1.8.2 IDE
/*
   (c)2014-2017 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This generated code may be freely used for both private and commercial use
   provided this copyright is maintained.
*/

// This sketch needs the BLEPeripheral library V0.4.0 from and compile using Arduino V1.8.2 IDE
// from https://github.com/sandeepmistry/arduino-BLEPeripheral
#include <BLEPeripheral.h>

// define pins for RedBeardLab BLE Nano (none used)
#define BLE_REQ   -1
#define BLE_RDY   -1
#define BLE_RST   -1

// download the pfodParser library V3.6+ from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
#include <pfodParser.h>
#include "RedbearNano_board.h"
#include "Nano_IO_Pin.h"
#include "IO_Selector.h"
#include "HL_Switch.h"
#include "Nano_ADC_3V3_Display.h"
#include "Help_hidable.h"
#include "Starter_Slider.h"

int swap01(int); // method prototype for slider end swaps

// =========== pfodBLESerial definitions
const char* localName = "Redbear BLE";  // <<<<<<  change this string to customize the adverised name of your board (max 20 chars)
class pfodBLESerial : public BLEPeripheral, public Stream {
  public:
    pfodBLESerial(unsigned char req, unsigned char rdy, unsigned char rst); void begin(); void poll(); size_t write(uint8_t); size_t write(const uint8_t*, size_t); int read();
    int available(); void flush(); int peek(); void close(); bool isConnected();
  private:
    const static uint8_t pfodEOF[1]; const static char* pfodCloseConnection;  static const int BLE_MAX_LENGTH = 20;
    static const int BLE_RX_MAX_LENGTH = 256; static volatile size_t rxHead; static volatile size_t rxTail;
    volatile static uint8_t rxBuffer[BLE_RX_MAX_LENGTH];  volatile static bool connected;
    size_t txIdx;  uint8_t txBuffer[BLE_MAX_LENGTH]; static void connectHandler(BLECentral& central);
    static void disconnectHandler(BLECentral& central); static void receiveHandler(BLECentral& central, BLECharacteristic& rxCharacteristic);
    static void addReceiveBytes(const uint8_t* bytes, size_t len);
    // ====  create Nordic UART service =========
    BLEService               uartService          = BLEService("6E400001B5A3F393E0A9E50E24DCCA9E");
    // create characteristic
    BLECharacteristic    txCharacteristic = BLECharacteristic("6E400003B5A3F393E0A9E50E24DCCA9E", BLENotify , BLE_MAX_LENGTH); // == RX on central (android app)
    BLECharacteristic    rxCharacteristic = BLECharacteristic("6E400002B5A3F393E0A9E50E24DCCA9E", BLEWrite, BLE_MAX_LENGTH);  // == TX on central (android app)
};
volatile size_t pfodBLESerial::rxHead = 0; volatile size_t pfodBLESerial::rxTail = 0;
volatile uint8_t pfodBLESerial::rxBuffer[BLE_RX_MAX_LENGTH]; const uint8_t pfodBLESerial::pfodEOF[1] = {(uint8_t) - 1};
const char* pfodBLESerial::pfodCloseConnection = "{!}"; volatile bool pfodBLESerial::connected = false;
// =========== end pfodBLESerial definitions
pfodParser parser("V7"); // create a parser to handle the pfod messages
pfodBLESerial bleSerial(BLE_REQ, BLE_RDY, BLE_RST); // create a BLE serial connection

pfodDwgs dwgs(&parser);  // drawing support
// Commands for drawing controls only need to unique per drawing, you can re-use cmds 'a' in another drawing.

const int refreshTime = 2000; // 2sec refresh when not editing to update ADC readings and input states
const int editRefreshTime = 0; // no refresh editing

IO_Selector ioSelector(false, &dwgs);
IO_Selector ioSelectorPWM(true, &dwgs);
Starter_Slider slider('P', &dwgs);
HL_Switch hlswitch('D', &dwgs);

int selectedIdx = -1; // -1 -> nothing selected

Help_hidable mainHelp('Z', &dwgs);
Help_hidable expandedHelp('Z', &dwgs); // different dwg so can reused cmd

const char expandCmd = 'x'; // used in dwg z only
int z_idx = 400 + pfodDwgs::reserveIdx(1); // expand square index. Add 400 to avoid all the board dwg z_idx's

int closeTouchZone_order = 700 + pfodDwgs::reserveIdx(4); // keep close on top
char closeCmd = 'X';

int loading_z_Idx = 800 + pfodDwgs::reserveIdx(3); // keep this on top of everything else

// dimensions are in 1/10"
const int colPadding = 11;
const int rowPadding = 4;
const float expandScaling = 2.5f;
int expandCol = colPadding + 7;
int expandRow = rowPadding + 5;
const int cols_board = RedbearNano_board::cols_Nano;
const int rows_board = RedbearNano_board::rows_Nano;
const int expandedRowSize = (int) (((float) rows_board) / 1.15);

const float expandedBoxCols = cols_board / expandScaling;
const float expandedBoxRows = expandedRowSize / expandScaling;

RedbearNano_board redbearNano_board(&dwgs);

Nano_IO_Pin d4Switch('a', 4, &dwgs);
Nano_IO_Pin d5Switch('b', 5, &dwgs);
Nano_IO_Pin d6Switch('c', 6, &dwgs);
Nano_IO_Pin d7Switch('d', 7, &dwgs);

const char ledCmd = 'l';

Nano_ADC_3V3_Display a3(A3, &dwgs);
Nano_ADC_3V3_Display a4(A4, &dwgs);
Nano_ADC_3V3_Display a5(A5, &dwgs);

const int switchArrayLen = 4;
const int analogArrayLen = 3;
Nano_IO_Pin* switchArray[switchArrayLen];
Nano_ADC_3V3_Display *analogArray[analogArrayLen];

void clearSelections() {
  selectedIdx = -1;
  for (int i = 0; i < switchArrayLen; i++) {
    // clear all selections
    switchArray[i]->clearSelected();
  }
}

// do this on connection
void forceFullUpdate() {
  clearSelections();
  for (int i = 0; i < switchArrayLen; i++) {
    switchArray[i]->forceFullUpdate();
  }
  for (int i = 0; i < analogArrayLen; i++) {
    analogArray[i]->skipUpdate();
  }
}

bool Nano_digitalPinHasPWM(int i) {
  return true;
}

void onConnectForceUpdates() {
  forceFullUpdate();
}

// the setup routine runs once on reset:
void setup() {
  // set advertised local name and service UUID
  // begin initialization
  bleSerial.begin();
  parser.connect(&bleSerial);

  // setup the positions of the drawing controls

  mainHelp.setBase(cols_board - 1.5, 1.5);
  mainHelp.setPopUpOffset(-5, 7);

  mainHelp.setBaseScaling(0.75);
  mainHelp.setLabel(F("<-2>Drag black outline\nto zoom in on the controls"));
  // 19 char max x 2 lines == ~ 38 x 12 height
  // round up to next even number
  mainHelp.setWidthHeight(30, 8);

  expandedHelp.setBase(cols_board - 1.5, 1.5);
  expandedHelp.setPopUpOffset(-5, 7); // location of bottom right of popup

  expandedHelp.setBaseScaling(0.75);
  expandedHelp.setLabel(F("<-2>Click Digital pin to configure.\nAnalog pins are display only."));
  // 19 char max x 2 lines == ~ 38 x 12 height
  // round up to next even number
  expandedHelp.setWidthHeight(30, 8);
  slider.setLabel(F("<+3><bk>PWM "));
  slider.setBase(10, 8);
  hlswitch.setBase(35, 6);
  hlswitch.setBaseScaling(1.5);

  redbearNano_board.setBase(0, 0);
  redbearNano_board.setLedCmd(ledCmd);

  d4Switch.setBase(8 + 3, RedbearNano_board::boardRows - 3);
  d5Switch.setBase(6 + 3, RedbearNano_board::boardRows - 3);
  d6Switch.setBase(4 + 3, RedbearNano_board::boardRows - 3);
  d7Switch.setBase(0 + 3, RedbearNano_board::boardRows - 3);


  int i = 0;
  switchArray[i++] = &d4Switch;
  switchArray[i++] = &d5Switch;
  switchArray[i++] = &d6Switch;
  switchArray[i++] = &d7Switch;

  a5.setLabel(F("A5"));

  a3.setBase(0, RedbearNano_board::boardRows - 5);
  a4.setBase(0, RedbearNano_board::boardRows - 3);
  a5.setBase(3, RedbearNano_board::boardRows + 1);

  i = 0;
  analogArray[i++] = &a3;
  analogArray[i++] = &a4;
  analogArray[i++] = &a5;

  pinMode(13, OUTPUT); // set led as output (default already?)
}

// if cols in expaned view == normal view then
// box size == cols/expandScale,
void addExpandBox() {
  dwgs.rectangle().idx(z_idx).centered().color(dwgs.BLACK)
  .size(expandedBoxCols, expandedBoxRows).offset(expandCol, expandRow).send();
}

void disableExpandControl() {
  dwgs.touchZone().cmd(expandCmd).size(cols_board, rows_board)//
  .filter(dwgs.TOUCH_DISABLED).send();
}

void addExpandControl() {
  addExpandBox();
  dwgs.touchZone().cmd(expandCmd).size(cols_board, rows_board)//
  .filter(dwgs.DOWN_UP).send();
  dwgs.touchAction()
  .cmd(expandCmd)
  .action( // create the action
    dwgs.rectangle().idx(z_idx).centered().color(dwgs.BLACK)
    .size(expandedBoxCols, expandedBoxRows)
    .offset(dwgs.TOUCHED_COL, dwgs.TOUCHED_ROW)//
  ).send();
}

int findSwitch(char dwgCmd) {
  for (int i = 0; i < switchArrayLen; i++) {
    if (switchArray[i]->getCmd() == dwgCmd) {
      return i;
    }
  }
  return -1; //not found
}

// digital pins 4 to 7
// led is 13 always output
// analog pins A3 to A6
void readDigitalInputs() {
  for (byte i = 0; i < switchArrayLen; i++) {
    byte mode = switchArray[i]->getMode();
    if ((mode == Nano_IO_Pin::DIGITAL_INPUT) || (mode == Nano_IO_Pin::DIGITAL_INPUT_PULLUP)) {
      int pinNo = switchArray[i]->getPinNo();
      switchArray[i]->setValue(digitalRead(pinNo));
    }
  }
  digitalWrite(13, redbearNano_board.getLed());
}

void readAnalogInputs() {
  for (byte i = 0; i < analogArrayLen; i++) {
    int pinNo = analogArray[i]->getPinNo();
   // parser.print("reading analog pin "); parser.print(pinNo); parser.print(':'); parser.println(analogRead(pinNo));
    analogArray[i]->setValue(analogRead(pinNo));
  }
}

void setReadDigitalPin(int idx) {
  if ((idx < 0) || (idx >= switchArrayLen)) {
    return;
  }
  int pinNo = switchArray[idx]->getPinNo();
  byte mode = switchArray[idx]->getMode();
  if (mode == Nano_IO_Pin::DIGITAL_INPUT) {
    pinMode(pinNo, INPUT);
    switchArray[idx]->setValue(digitalRead(pinNo));
  } else if (mode == Nano_IO_Pin::DIGITAL_INPUT_PULLUP) {
    pinMode(pinNo, INPUT_PULLUP);
    switchArray[idx]->setValue(digitalRead(pinNo));
  } else if (mode == Nano_IO_Pin::DIGITAL_OUTPUT) { // digital output
    pinMode(pinNo, OUTPUT);
    digitalWrite(pinNo, switchArray[idx]->getValue());
  } else { // PWM
    pinMode(pinNo, OUTPUT);
    analogWrite(pinNo, switchArray[idx]->getPwmValue());
  }
  digitalWrite(13, redbearNano_board.getLed());
}

// the loop routine runs over and over again forever:
void loop() {
  uint8_t cmd = parser.parse(); // parse incoming data from connection
  // parser returns non-zero when a pfod command is fully parsed
  if (cmd != 0) { // have parsed a complete msg { to }
    uint8_t* pfodFirstArg = parser.getFirstArg(); // may point to \0 if no arguments in this msg.
    pfod_MAYBE_UNUSED(pfodFirstArg); // may not be used, just suppress warning
    long pfodLongRtn; // used for parsing long return arguments, if any
    pfod_MAYBE_UNUSED(pfodLongRtn); // may not be used, just suppress warning
    if ('.' == cmd) {
      // pfodApp has connected and sent {.} , it is asking for the main menu
      // read digital input and analog values here for refresh
      readDigitalInputs();
      readAnalogInputs();
      clearSelections();
      if (!parser.isRefresh()) {
        sendMainMenu(); // send back the menu designed
      } else {
        sendMainMenuUpdate(); // menu is cached just send update
      }

      // now handle commands returned from button/vsliders
    } else if ('A' == cmd) { // user pressed menu item that loaded drawing with load cmd 'z'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (mainHelp.getCmd() == dwgCmd) { //
        dwgs.startUpdate();
        addExpandBox();
        dwgs.end();
      } else if (expandCmd == dwgCmd) { //
        int col = parser.getTouchedCol();
        int row = parser.getTouchedRow();
        expandCol = col;
        expandRow = row;
        showExpandedViewUpdate();
      } else {
        parser.print(F("{+}"));
      }
      // always send back a response or pfodApp will timeout

    } else if ('X' == cmd) { // user pressed expanded drawing loaded with load cmd 'x'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      // can be \0 if no dwg or no active areas defined
      if (expandedHelp.getCmd() == dwgCmd) { //
        dwgs.startUpdate(); // empty update to close help
        dwgs.end();
      } else if (closeCmd == dwgCmd) {
        closeExpandedView();
      } else if (ledCmd == dwgCmd) {
        redbearNano_board.setLed((byte) ((redbearNano_board.getLed() == 0) ? 1 : 0)); // toggle
        digitalWrite(13, redbearNano_board.getLed());
        sendDrawingUpdates_x();
      } else {
        int idx = findSwitch(dwgCmd);
        if (idx >= 0) {
          if (selectedIdx != idx) {
            if (selectedIdx >= 0) {
              switchArray[selectedIdx]->clearSelected();
            }
            if (idx < 0) {
              // skip Rx,Tx
              closeIOSelection();
            } else {
              selectedIdx = idx;
              switchArray[selectedIdx]->setSelected();
              ioSelector.setValue(switchArray[selectedIdx]->getMode());
              ioSelectorPWM.setValue(switchArray[selectedIdx]->getMode());
              ioSelector.forceFullUpdate();
              ioSelectorPWM.forceFullUpdate();
              slider.setValue(switchArray[selectedIdx]->getPwmValue());
              hlswitch.setValue(switchArray[selectedIdx]->getValue());
              hlswitch.forceFullUpdate();
              if (switchArray[selectedIdx]->getMode() == Nano_IO_Pin::DIGITAL_PWM) {
                showIOSelectionPWM();
              } else if (switchArray[selectedIdx]->getMode() == Nano_IO_Pin::DIGITAL_OUTPUT) {
                if (Nano_digitalPinHasPWM(selectedIdx)) {
                  showIOSelectionOutputPWM();
                } else {
                  showIOSelectionOutput();
                }
              } else { // INPUT
                if (Nano_digitalPinHasPWM(selectedIdx)) {
                  showIOSelectionInputPWM();
                } else {
                  showIOSelectionInput();
                }
              }
            }
          } else if (selectedIdx == idx) {
            // pressed again
            // just close
            closeIOSelection();
          } else {
            // clear selection
            sendMainMenuUpdate();
          }
        } else {
          sendDrawingUpdates_x();
        }
      }

    } else if ('B' == cmd) { // user pressed expanded drawing loaded with load cmd 'b'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (closeCmd == dwgCmd) {
        closeIOSelection();
      } else {
        if (ioSelector.getCmd_Input() == dwgCmd) {
          ioSelector.setValue(Nano_IO_Pin::DIGITAL_INPUT);
        } else if (ioSelector.getCmd_InputPullup() == dwgCmd) {
          ioSelector.setValue(Nano_IO_Pin::DIGITAL_INPUT_PULLUP);
        } else if (ioSelector.getCmd_Output() == dwgCmd) {
          ioSelector.setValue(Nano_IO_Pin::DIGITAL_OUTPUT);
        } else if (ioSelector.getCmd_PWM() == dwgCmd) {
          ioSelector.setValue(Nano_IO_Pin::DIGITAL_PWM);
        }
        if (selectedIdx >= 0) {
          switchArray[selectedIdx]->setMode(ioSelector.getValue());
          setReadDigitalPin(selectedIdx);
        }

        if (ioSelector.getValue() == Nano_IO_Pin::DIGITAL_PWM) {
          showIOSelectionPWM();
        } else if (ioSelector.getValue() == Nano_IO_Pin::DIGITAL_OUTPUT) {
          showIOSelectionOutput();
        } else {
          closeIOSelection();
        }
      }
      // always send back a response or pfodApp will timeout

    } else if ('P' == cmd) { // user pressed expanded drawing loaded with load cmd 'p'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (closeCmd == dwgCmd) {
        closeIOSelection();
      } else {
        if (ioSelectorPWM.getCmd_Input() == dwgCmd) { // user touched LED control
          ioSelectorPWM.setValue(Nano_IO_Pin::DIGITAL_INPUT);
        } else if (ioSelectorPWM.getCmd_InputPullup() == dwgCmd) { // user touched LED control
          ioSelectorPWM.setValue(Nano_IO_Pin::DIGITAL_INPUT_PULLUP);
        } else if (ioSelector.getCmd_Output() == dwgCmd) { // user touched LED control
          ioSelectorPWM.setValue(Nano_IO_Pin::DIGITAL_OUTPUT);
        } else if (ioSelectorPWM.getCmd_PWM() == dwgCmd) { // user touched LED control
          ioSelectorPWM.setValue(Nano_IO_Pin::DIGITAL_PWM);
        }
        if (selectedIdx >= 0) {
          switchArray[selectedIdx]->setMode(ioSelectorPWM.getValue());
          setReadDigitalPin(selectedIdx);
        }

        if (ioSelectorPWM.getValue() == Nano_IO_Pin::DIGITAL_PWM) {
          showIOSelectionPWM();
        } else if (ioSelectorPWM.getValue() == Nano_IO_Pin::DIGITAL_OUTPUT) {
          showIOSelectionOutputPWM();
        } else {
          closeIOSelection();
        }
      }
      // always send back a response or pfodApp will timeout

    } else if ('C' == cmd) { // user pressed expanded drawing loaded with load cmd 'c'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (closeCmd == dwgCmd) {
        closeIOSelection(); // close both
        // skip update
      } else if (slider.getCmd() == dwgCmd) {
        slider.setValue(parser.getTouchedCol());
        switchArray[selectedIdx]->setPwmValue(slider.getValue());
        setReadDigitalPin(selectedIdx);
        refreshPWM();
      } else {
        parser.print(F("{+}")); // remove any help bubble
      }
    } else if ('D' == cmd) { // user pressed expanded drawing loaded with load cmd 'd'
      char dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (closeCmd == dwgCmd) {
        closeIOSelection(); // close both
      } else if (hlswitch.getCmd() == dwgCmd) {
        hlswitch.setValue((hlswitch.getValue() == 0) ? 1 : 0);
        switchArray[selectedIdx]->setValue(hlswitch.getValue());
        setReadDigitalPin(selectedIdx);
        refreshOutput();
      } else {
        parser.print(F("{+}")); // remove any help bubble
      }

    } else if ('b' == cmd) { // pfodApp is asking to load dwg 'b'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        sendDrawing_b();
      } else { // refresh just update drawing state
        sendDrawingUpdates_b();
      }
    } else if ('p' == cmd) { // pfodApp is asking to load dwg 'p'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        sendDrawing_p();
      } else { // refresh just update drawing state
        sendDrawingUpdates_p();
      }

    } else if ('c' == cmd) { // pfodApp is asking to load dwg 'c'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        sendDrawing_c();
      } else { // refresh just update drawing state
        sendDrawingUpdates_c();
      }

    } else if ('d' == cmd) { // pfodApp is asking to load dwg 'd'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        sendDrawing_d();
      } else { // refresh just update drawing state
        sendDrawingUpdates_d();
      }

    } else if ('a' == cmd) { // pfodApp is asking to load dwg 'a'
      // dwg 'a' is the main board layout and is loaded in 8 parts
      if (!parser.isRefresh()) { // not refresh send whole dwg
        // get starting offset
        parser.parseLong(pfodFirstArg, &pfodLongRtn); // get starting offset
        long startingOffset = pfodLongRtn;
        if (startingOffset == 0) {
          sendDrawing_a0();
        } else if (startingOffset == 1) {
          sendDrawing_a1();
        } else if (startingOffset == 2) {
          sendDrawing_a2();
        } else if (startingOffset == 3) {
          sendDrawing_a3();
        } else if (startingOffset == 4) {
          sendDrawing_a4();
        } else if (startingOffset == 5) {
          sendDrawing_a5();
        } else if (startingOffset == 6) {
          sendDrawing_a6();
        } else {
          parser.print(F("{}")); // finished
        }
      } else { // refresh just update drawing state
        sendDrawingUpdates_a();
      }

    } else if ('x' == cmd) { // pfodApp is asking to load dwg 'x'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        sendDrawing_x();
      } else { // refresh just update drawing state
        sendDrawingUpdates_x();
      }

    } else if ('z' == cmd) { // pfodApp is asking to load dwg 'z'
      if (!parser.isRefresh()) { // not refresh send whole dwg
        sendDrawing_z();
      } else { // refresh just update drawing state
        sendDrawingUpdates_z();
      }

    } else if ('!' == cmd) {
      onConnectForceUpdates();
      // CloseConnection command
      closeConnection(parser.getPfodAppStream());
    } else {
      // unknown command
      parser.print(F("{}")); // always send back a pfod msg otherwise pfodApp will disconnect.
    }
  }
  //  <<<<<<<<<<<  Your other loop() code goes here

}

void closeConnection(Stream * io) {
  // add any special code here to force connection to be dropped
  ((pfodBLESerial*)io)->close();
}

void addCloseButton(double c, double r, double scale) {
  dwgs.pushZero(c, r, scale);
  dwgs.circle().filled().idx(closeTouchZone_order).radius(1.8).offset(0, 0).color(dwgs.BLACK).send();
  dwgs.circle().filled().idx(closeTouchZone_order + 1).radius(1.5).offset(0, 0).color(dwgs.WHITE)
  .send();
  dwgs.label().idx(closeTouchZone_order + 2).text(F("x")).color(dwgs.BLACK).offset(0, -0.5).send();
  dwgs.touchZone().idx(closeTouchZone_order + 3).cmd(closeCmd).size(0.01, 0.01).send();
  dwgs.touchAction().cmd(closeCmd).action( //
    dwgs.circle().filled().idx(closeTouchZone_order + 1).radius(1.5).offset(0, 0).color(dwgs.BLACK))
  .send();
  dwgs.popZero();
}

void showExpandedViewUpdate() {
  clearSelections();
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  parser.print(F("|A"));
  parser.print(F("|X"));
  parser.print(F("|B-"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}


void closeExpandedView() {
  clearSelections();
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(refreshTime);
  // send menu items
  parser.print(F("|X-"));
  parser.print(F("|A"));
  parser.print(F("|B-"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void sendDrawing_d() {
  dwgs.start(50, 12); // background defaults to WHITE if omitted
  parser.sendVersion(); // send the parser version to cache this image
  dwgs.label().fontSize(+3).text(F("Output")).offset(15, 6).send();
  hlswitch.draw(); // draw the control
  dwgs.popZero();
  dwgs.end();
}

void sendDrawingUpdates_d() {
  dwgs.startUpdate();
  hlswitch.update(); // update with current state
  dwgs.end();
}

void sendDrawing_c() {
  dwgs.start(50, 13); // background defaults to WHITE if omitted
  parser.sendVersion(); // send the parser version to cache this image
  slider.draw(); // draw the control
  dwgs.popZero();
  dwgs.end();
}

void sendDrawingUpdates_c() {
  dwgs.startUpdate();
  slider.update(); // update with current state
  dwgs.end();
}

void sendDrawing_b() {
  dwgs.start(50, 15); // background defaults to WHITE if omitted
  parser.sendVersion(); // send the parser version to cache this image
  dwgs.pushZero(0, 1);
  ioSelector.draw(); // draw the control
  dwgs.popZero();
  addCloseButton(50 - 2, 15 - 2, 1);
  dwgs.end();
}

void sendDrawingUpdates_b() {
  dwgs.startUpdate();
  ioSelector.update(); // update with current state
  dwgs.end();
}

void sendDrawing_p() {
  dwgs.start(50, 15); // background defaults to WHITE if omitted
  parser.sendVersion(); // send the parser version to cache this image
  dwgs.pushZero(0, 1);
  ioSelectorPWM.draw(); // draw the control
  dwgs.popZero();
  addCloseButton(50 - 2, 15 - 2, 1);
  dwgs.end();
}

void sendDrawingUpdates_p() {
  dwgs.startUpdate();
  ioSelectorPWM.update(); // update with current state
  dwgs.end();
}

void showIOSelectionInput() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void showIOSelectionInputPWM() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B-"));
  parser.print(F("|P"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void showIOSelectionPWM() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|P"));
  parser.print(F("|B-"));
  parser.print(F("|C"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void showIOSelectionOutput() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D"));
  parser.print(F("}")); // close pfod message
}

void showIOSelectionOutputPWM() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B-"));
  parser.print(F("|P"));
  parser.print(F("|C-"));
  parser.print(F("|D"));
  parser.print(F("}")); // close pfod message
}

void closeIOSelection() {
  clearSelections();
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|X"));
  parser.print(F("|B-"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}



void sendMainMenu() {
  clearSelections();
  // !! Remember to change the parser version string
  // every time you edit this method
  parser.print(F("{,")); // start a Menu screen pfod message
  // send menu background, format, prompt, refresh and version
  parser.print(F("~`"));
  parser.print(refreshTime);
  parser.sendVersion(); // send the menu version
  // send menu items
  parser.print(F("|+X-~x"));
  parser.print(F("|+B-~b"));
  parser.print(F("|+P-~p"));
  parser.print(F("|+C-~c"));
  parser.print(F("|+D-~d"));
  parser.print(F("|+A~z")); // change to embedded
  parser.print(F("}")); // close pfod message
}

void sendMainMenuUpdate() {
  clearSelections();
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(refreshTime);
  // send menu items
  parser.print(F("|A"));
  parser.print(F("|X-"));
  parser.print(F("|B-"));
  parser.print(F("|P-"));
  parser.print(F("|C-"));
  parser.print(F("|D-"));
  parser.print(F("}")); // close pfod message
}

void refreshOutput() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|A")); // reloads z
  parser.print(F("|D")); // reloads pwm slider
  parser.print(F("}")); // close pfod message
}

void refreshPWM() {
  parser.print(F("{;`")); // start an Update Menu pfod message
  parser.print(editRefreshTime);
  // send menu items
  parser.print(F("|A")); // reloads z
  parser.print(F("|C")); // reloads pwm slider
  parser.print(F("}")); // close pfod message
}


void sendDrawing_x() {
  dwgs.start(cols_board, expandedRowSize, dwgs.BLUE);
  parser.sendVersion();
  expandedHelp.draw();
  dwgs.pushZero(cols_board / 2.0f, expandedRowSize / 2.0f, expandScaling - 0.1);
  dwgs.insertDwg().loadCmd('a').offset(expandCol, expandRow).send(); // offset in dwg being placed
  dwgs.popZero();
  addCloseButton(cols_board - 1.15, expandedRowSize - 1.15, 0.58);
  dwgs.end();
}

void sendDrawingUpdates_x() {
  dwgs.startUpdate();
  dwgs.insertDwg().loadCmd('a').offset(expandCol, expandRow).send(); // offset in dwg being placed
  dwgs.end();
}

bool loadingShowing = false;
void sendDrawing_z() {
  dwgs.start(cols_board, rows_board, dwgs.BLUE);
  parser.sendVersion();
  mainHelp.draw();//
  mainHelp.hide();// hide it
  disableExpandControl();

  dwgs.insertDwg().loadCmd('a').offset(0, 0).send(); // offset in dwg being placed i.e. topleft
  dwgs.end();
}

void sendDrawingUpdates_z() {
  dwgs.startUpdate();
  dwgs.insertDwg().loadCmd('a').offset(0, 0).send(); // offset in dwg being placed i.e. topleft
  addExpandControl();
  mainHelp.unhide();
  dwgs.end();
}

void sendDrawing_a0() {
  // send outline of board
  dwgs.start(cols_board, rows_board, dwgs.BLUE, true);
  parser.sendVersion();
  draw_a0();
  dwgs.end();
}

void sendDrawing_a1() {
  dwgs.startUpdate(true);
  draw_a1();
  dwgs.end();
}

void sendDrawing_a2() {
  dwgs.startUpdate(true);
  draw_a2();
  dwgs.end();
}

void sendDrawing_a3() {
  dwgs.startUpdate(true);
  draw_a3();
  dwgs.end();
}

void sendDrawing_a4() {
  dwgs.startUpdate(true);
  draw_a4();
  dwgs.end();
}

void sendDrawing_a5() {
  dwgs.startUpdate(true);
  draw_a5();
  dwgs.end();
}

void sendDrawing_a6() {
  dwgs.startUpdate(); // final part no more to come
  draw_a6();
  dwgs.end();
}

void draw_a0() {
  redbearNano_board.drawFirst();//
  redbearNano_board.drawOutline();
  loadingShowing = true;
  dwgs.label().idx(loading_z_Idx + 2).fontSize(-5).bold().color(dwgs.WHITE).text(F("       Loading")).italic()
  .offset(RedbearNano_board::boardCols / 2, 6.5).send();
  dwgs.label().idx(loading_z_Idx + 1).fontSize(-5).bold().color(dwgs.WHITE).text(F("Board Layout"))
  .offset(RedbearNano_board::boardCols / 2, 8.5).italic().send();
  dwgs.pushZero(RedbearNano_board::boardCols / 2, 9.25); // avoids having to send offset for each updates to this
  dwgs.label().idx(loading_z_Idx).fontSize(-5).bold().color(dwgs.WHITE).text(F("......")).send();
  dwgs.popZero();
}

void draw_a1() {
  dwgs.label().idx(loading_z_Idx).fontSize(-5).bold().color(dwgs.WHITE).text(F(" .....")).send();
  redbearNano_board.drawPins();
}

void draw_a2() {
  dwgs.label().idx(loading_z_Idx).fontSize(-5).bold().color(dwgs.WHITE).text(F("  ....")).send();
  drawNames();
}

void draw_a3() {
  dwgs.label().idx(loading_z_Idx).fontSize(-5).bold().color(dwgs.WHITE).text(F("   ...")).send();
  drawDs4_5();

}

void draw_a4() {
  dwgs.label().idx(loading_z_Idx).fontSize(-5).bold().color(dwgs.WHITE).text(F("    ..")).send();
  drawDs6_7();
}

void draw_a5() {
  dwgs.label().idx(loading_z_Idx).fontSize(-5).bold().color(dwgs.WHITE).text(F("     .")).send();
  drawAs();
}

void draw_a6() {
  drawingUpdates(); // do full update here to update rest of the pins
  redbearNano_board.drawLast();//
}

void drawAs() {
  dwgs.pushZero(-colPadding - 1.75, -1.75);
  a3.draw();
  a4.draw();
  dwgs.popZero();
  dwgs.pushZero(0.5, 0.5);
  a5.draw();
  dwgs.popZero();
}

void drawNames() {
  redbearNano_board.drawLeftNames();
  redbearNano_board.drawRightNames();
  redbearNano_board.drawBottomNames();
  redbearNano_board.drawLedTxt();
}

void drawDs4_5() {
  d4Switch.draw();
  d5Switch.draw();
}

void drawDs6_7() {
  d6Switch.draw();
  d7Switch.draw();
}

void updateDs() {
  d4Switch.update();
  d5Switch.update();
  d6Switch.update();
  d7Switch.update();
}

void updateAs() {
  a3.update();
  a4.update();
  a5.update();
}

void drawingUpdates() {
  redbearNano_board.update();// handles LED
  updateDs();
  updateAs();
}

void sendDrawingUpdates_a() {
  dwgs.startUpdate();
  if (loadingShowing) {
    dwgs.hide().idx(loading_z_Idx).send();
    dwgs.hide().idx(loading_z_Idx + 1).send();
    dwgs.hide().idx(loading_z_Idx + 2).send();
    loadingShowing = false;
  }
  drawingUpdates();
  dwgs.end();
}


// ========== pfodBLESerial methods
pfodBLESerial::pfodBLESerial(unsigned char req, unsigned char rdy, unsigned char rst) : BLEPeripheral(req, rdy, rst) {
  setLocalName(localName);  addAttribute(uartService);
  setAdvertisedServiceUuid(uartService.uuid());
  addAttribute(rxCharacteristic);
  rxCharacteristic.setEventHandler(BLEWritten, pfodBLESerial::receiveHandler);
  setEventHandler(BLEConnected, pfodBLESerial::connectHandler);  setEventHandler(BLEDisconnected, pfodBLESerial::disconnectHandler);
  addAttribute(txCharacteristic);
};

bool pfodBLESerial::isConnected() {
  return (connected && txCharacteristic.subscribed());
}
void pfodBLESerial::begin() {
  BLEPeripheral::begin();
}

void pfodBLESerial::close() {
  BLEPeripheral::disconnect();
}

void pfodBLESerial::poll() {
  BLEPeripheral::poll();
}

size_t pfodBLESerial::write(const uint8_t* bytes, size_t len) {
  for (size_t i = 0; i < len; i++) {
    write(bytes[i]);
  }
  return len; // just assume it is all written
}

size_t pfodBLESerial::write(uint8_t b) {
  BLEPeripheral::poll();
  delay(10); // <<<<<<<<<<<<<< this delay seems to make the writes reliable
  if (!isConnected()) {
    return 1;
  }
  txBuffer[txIdx++] = b;
  if ((txIdx == sizeof(txBuffer)) || (b == ((uint8_t)'\n')) || (b == ((uint8_t)'}')) ) {
    flush(); // send this buffer if full or end of msg or rawdata newline
  }
  return 1;
}

int pfodBLESerial::read() {
  if (rxTail == rxHead) {
    return -1;
  }
  // note increment rxHead befor writing
  // so need to increment rxTail befor reading
  rxTail = (rxTail + 1) % sizeof(rxBuffer);
  uint8_t b = rxBuffer[rxTail];
  return b;
}

// called as part of parser.parse() so will poll() each loop()
int pfodBLESerial::available() {
  BLEPeripheral::poll();
  flush(); // send any pending data now. This happens at the top of each loop()
  int rtn = ((rxHead + sizeof(rxBuffer)) - rxTail ) % sizeof(rxBuffer);
  return rtn;
}

void pfodBLESerial::flush() {
  if (txIdx == 0) {
    return;
  }
  txCharacteristic.setValue(txBuffer, txIdx);
  txIdx = 0;
  BLEPeripheral::poll();
}

int pfodBLESerial::peek() {
  BLEPeripheral::poll();
  if (rxTail == rxHead) {
    return -1;
  }
  size_t nextIdx = (rxTail + 1) % sizeof(rxBuffer);
  uint8_t byte = rxBuffer[nextIdx];
  return byte;
}

void pfodBLESerial::connectHandler(BLECentral& central) {
  // clear parser with -1 incase partial message left
  // should not be one
  addReceiveBytes(pfodEOF, sizeof(pfodEOF));
  connected = true;
}

void pfodBLESerial::disconnectHandler(BLECentral& central) {
  // parser.closeConnection();
  // clear parser with -1 and insert {!} incase connection just lost
  addReceiveBytes(pfodEOF, sizeof(pfodEOF));
  addReceiveBytes((const uint8_t*)pfodCloseConnection, sizeof(pfodCloseConnection));
  connected = false;
}

void pfodBLESerial::addReceiveBytes(const uint8_t* bytes, size_t len) {
  // note increment rxHead befor writing
  // so need to increment rxTail befor reading
  for (size_t i = 0; i < len; i++) {
    rxHead = (rxHead + 1) % sizeof(rxBuffer);
    rxBuffer[rxHead] = bytes[i];
  }
}

void pfodBLESerial::receiveHandler(BLECentral& central, BLECharacteristic& rxCharacteristic) {
  size_t len = rxCharacteristic.valueLength();
  const unsigned char *data = rxCharacteristic.value();
  addReceiveBytes((const uint8_t*)data, len);
}
//======================= end pfodBLESerial methods

int swap01(int in) {
  return (in == 0) ? 1 : 0;
}
// ============= end generated code =========

